#!/usr/bin/env python
# -*- coding: utf-8 -*-

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   The GPL v3 licence is available at: https://www.gnu.org/licenses/gpl-3.0.en.html


# Ver.1   init relase by Brian Andersen. Script Based On Conbagui's Gold Coin Tutorial. https://gimplearn.net/viewtopic.php?f=3&t=744&hilit=Gold+Coin+Tutorial
# Ver.2   MareroQ: add text-along-path-04 by Ofnuts, replace "Clipboard" to "Clipboard Image"
# Ver.3   MareroQ: add: national characters support (utf-8), creating paths by plugin (no need to import)
# Ver.4   MareroQ: add options: "Make stars?", unnecessary brushes have been removed, cosmetic with sliders and font name
# Ver.5   MareroQ: add options: make center symbol (accepts UNICODE characters)
#                  https://www.gimpscripts.net/2020/09/baconbaguicoin-ver5-new.html
# Ver.6   MareroQ: add options: Bottom text type: "From Left to Right", "From Right to Left","Only Top Text in a Circle"
# Ver.7.0 MareroQ:/2021/12/11/ add preview, regulation of coin size and other parameters (Rim, Grooves,Ring, Stars)
#                  add add dial-marks-0.2.py by Ofnuts for create Rim & Grooves

#####################################################################################################################################################
############################################# START fu_preview 0.1.1 by Claude Lion $ J.F. Garcia ###################################################
#####################################################################################################################################################

import string as _string
import math
import gimp
import gimpcolor
import re
from gimpenums import *
pdb = gimp.pdb
import os, time, sys
import gettext
t = gettext.translation("gimp20-python", gimp.locale_directory, fallback=True)
_ = t.ugettext

class error(RuntimeError): pass
class CancelError(RuntimeError): pass

PF_INT8        = PDB_INT8
PF_INT16       = PDB_INT16
PF_INT32       = PDB_INT32
PF_INT         = PF_INT32
PF_FLOAT       = PDB_FLOAT
PF_STRING      = PDB_STRING
PF_VALUE       = PF_STRING
PF_COLOR       = PDB_COLOR
PF_COLOUR      = PF_COLOR
PF_ITEM        = PDB_ITEM
PF_DISPLAY     = PDB_DISPLAY
PF_IMAGE       = PDB_IMAGE
PF_LAYER       = PDB_LAYER
PF_CHANNEL     = PDB_CHANNEL
PF_DRAWABLE    = PDB_DRAWABLE
PF_VECTORS     = PDB_VECTORS

PF_TOGGLE      = 1000
PF_BOOL        = PF_TOGGLE
PF_SLIDER      = 1001
PF_SPINNER     = 1002
PF_ADJUSTMENT  = PF_SPINNER

PF_FONT        = 1003
PF_FILE        = 1004
PF_BRUSH       = 1005
PF_PATTERN     = 1006
PF_GRADIENT    = 1007
PF_RADIO       = 1008
PF_TEXT        = 1009
PF_PALETTE     = 1010
PF_FILENAME    = 1011
PF_DIRNAME     = 1012
PF_OPTION      = 1013
PF_TAB         = 1014
PF_DRAW        = 1015
PF_TITLE       = 1016

_type_mapping = {
    PF_INT8        : PDB_INT8,
    PF_INT16       : PDB_INT16,
    PF_INT32       : PDB_INT32,
    PF_FLOAT       : PDB_FLOAT,
    PF_STRING      : PDB_STRING,
    PF_COLOR       : PDB_COLOR,
    PF_ITEM        : PDB_ITEM,
    PF_DISPLAY     : PDB_DISPLAY,
    PF_IMAGE       : PDB_IMAGE,
    PF_LAYER       : PDB_LAYER,
    PF_CHANNEL     : PDB_CHANNEL,
    PF_DRAWABLE    : PDB_DRAWABLE,
    PF_VECTORS     : PDB_VECTORS,

    PF_TOGGLE      : PDB_INT32,
    PF_SLIDER      : PDB_FLOAT,
    PF_SPINNER     : PDB_INT32,

    PF_FONT        : PDB_STRING,
    PF_FILE        : PDB_STRING,
    PF_BRUSH       : PDB_STRING,
    PF_PATTERN     : PDB_STRING,
    PF_GRADIENT    : PDB_STRING,
    PF_RADIO       : PDB_STRING,
    PF_TEXT        : PDB_STRING,
    PF_PALETTE     : PDB_STRING,
    PF_FILENAME    : PDB_STRING,
    PF_DIRNAME     : PDB_STRING,
    PF_OPTION      : PDB_INT32,
    PF_TAB         : PDB_STRING,
    PF_DRAW        : PDB_INT32,
    PF_TITLE       : PDB_STRING,
}

_obj_mapping = {
    PF_INT8        : int,
    PF_INT16       : int,
    PF_INT32       : int,
    PF_FLOAT       : float,
    PF_STRING      : str,
    PF_COLOR       : gimpcolor.RGB,
    PF_ITEM        : int,
    PF_DISPLAY     : gimp.Display,
    PF_IMAGE       : gimp.Image,
    PF_LAYER       : gimp.Layer,
    PF_CHANNEL     : gimp.Channel,
    PF_DRAWABLE    : gimp.Drawable,
    PF_VECTORS     : gimp.Vectors,

    PF_TOGGLE      : bool,
    PF_SLIDER      : float,
    PF_SPINNER     : int,

    PF_FONT        : str,
    PF_FILE        : str,
    PF_BRUSH       : str,
    PF_PATTERN     : str,
    PF_GRADIENT    : str,
    PF_RADIO       : str,
    PF_TEXT        : str,
    PF_PALETTE     : str,
    PF_FILENAME    : str,
    PF_DIRNAME     : str,
    PF_OPTION      : int,
    PF_TAB         : str,
    PF_DRAW        : int,
    PF_TITLE       : str,
}

_registered_plugins_ = {}


def register(proc_name, blurb, help, author, copyright, date, label,
             imagetypes, params, results, function,
             menu=None, domain=None, on_query=None, on_run=None):
    """This is called to register a new plug-in."""

    # First perform some sanity checks on the data
    def letterCheck(str):
        allowed = _string.letters + _string.digits + "_" + "-"
        for ch in str:
            if not ch in allowed:
                return 0
        else:
            return 1

    if not letterCheck(proc_name):
        raise error, "procedure name contains illegal characters"

    for ent in params:
        if len(ent) < 4:
            raise error, ("parameter definition must contain at least 4 "
                          "elements (%s given: %s)" % (len(ent), ent))

        if type(ent[0]) != int:
            raise error, "parameter types must be integers"

        if not letterCheck(ent[1]):
            raise error, "parameter name contains illegal characters"

    for ent in results:
        if len(ent) < 3:
            raise error, ("result definition must contain at least 3 elements "
                          "(%s given: %s)" % (len(ent), ent))

        if type(ent[0]) != type(42):
            raise error, "result types must be integers"

        if not letterCheck(ent[1]):
            raise error, "result name contains illegal characters"

    plugin_type = PLUGIN

    if (not proc_name.startswith("python-") and
        not proc_name.startswith("python_") and
        not proc_name.startswith("extension-") and
        not proc_name.startswith("extension_") and
        not proc_name.startswith("plug-in-") and
        not proc_name.startswith("plug_in_") and
        not proc_name.startswith("file-") and
        not proc_name.startswith("file_")):
           proc_name = "python-fu-" + proc_name

    # if menu is not given, derive it from label
    need_compat_params = False
    if menu is None and label:
        fields = label.split("/")
        if fields:
            label = fields.pop()
            menu = "/".join(fields)
            need_compat_params = True

            import warnings
            message = ("%s: passing the full menu path for the menu label is "
                       "deprecated, use the 'menu' parameter instead"
                       % (proc_name))
            warnings.warn(message, DeprecationWarning, 3)

        if need_compat_params and plugin_type == PLUGIN:
            file_params = [(PDB_STRING, "filename", "The name of the file", ""),
                           (PDB_STRING, "raw-filename", "The name of the file", "")]

            if menu is None:
                pass
            elif menu.startswith("<Load>"):
                params[0:0] = file_params
            elif menu.startswith("<Image>") or menu.startswith("<Save>"):
                params.insert(0, (PDB_IMAGE, "image", "Input image", None))
                params.insert(1, (PDB_DRAWABLE, "drawable", "Input drawable", None))
                if menu.startswith("<Save>"):
                    params[2:2] = file_params

    _registered_plugins_[proc_name] = (blurb, help, author, copyright,
                                       date, label, imagetypes,
                                       plugin_type, params, results,
                                       function, menu, domain,
                                       on_query, on_run)

def _query():
    for plugin in _registered_plugins_.keys():
        (blurb, help, author, copyright, date,
         label, imagetypes, plugin_type,
         params, results, function, menu, domain,
         on_query, on_run) = _registered_plugins_[plugin]

        def make_params(params):
            return [(_type_mapping[x[0]],
                     x[1],
                     _string.replace(x[2], "_", "")) for x in params]

        params = make_params(params)
        # add the run mode argument ...
        params.insert(0, (PDB_INT32, "run-mode",
                                     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }"))

        results = make_params(results)

        if domain:
            try:
                (domain, locale_dir) = domain
                gimp.domain_register(domain, locale_dir)
            except ValueError:
                gimp.domain_register(domain)

        gimp.install_procedure(plugin, blurb, help, author, copyright,
                               date, label, imagetypes, plugin_type,
                               params, results)

        if menu:
            gimp.menu_register(plugin, menu)
        if on_query:
            on_query()

def getXtra(xtra, valName, defValue):
    p = defValue
    if xtra.get(valName):
        p = xtra.get(valName)
    return p

def _get_defaults(proc_name):
    import gimpshelf

    (blurb, help, author, copyright, date,
     label, imagetypes, plugin_type,
     params, results, function, menu, domain,
     on_query, on_run) = _registered_plugins_[proc_name]

    key = "python-fu-save--" + proc_name

    if gimpshelf.shelf.has_key(key):
        return gimpshelf.shelf[key]
    else:
        # return the default values
        return [x[3] for x in params]

def _set_defaults(proc_name, defaults):
    import gimpshelf

    key = "python-fu-save--" + proc_name
    gimpshelf.shelf[key] = defaults

def _interact(proc_name, start_params):
    (blurb, help, author, copyright, date, label, imagetypes, plugin_type, params, results, function, menu, domain, on_query, on_run) = _registered_plugins_[proc_name]

    def run_script(run_params):
        params = start_params + tuple(run_params)
        _set_defaults(proc_name, params)
        return apply(function, params)

    params = params[len(start_params):]

    # short circuit for no parameters ...
    if len(params) == 0:
         return run_script([])

    import pygtk
    pygtk.require('2.0')

    import gimpui, gtk
#    import pango
    gimpui.gimp_ui_init () # add MrQ
    defaults = _get_defaults(proc_name)
    defaults = defaults[len(start_params):]

    class EntryValueError(Exception):
        pass

    def warning_dialog(parent, primary, secondary=None):
        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING, gtk.BUTTONS_CLOSE, primary)
        if secondary:
            dlg.format_secondary_text(secondary)
        dlg.run()
        dlg.destroy()

    def error_dialog(parent, proc_name):
        import sys, traceback

        exc_str = exc_only_str = _("Missing exception information")
        try:
            etype, value, tb = sys.exc_info()
            exc_str = "".join(traceback.format_exception(etype, value, tb))
            exc_only_str = "".join(traceback.format_exception_only(etype, value))
        finally:
            etype = value = tb = None

        title = _("An error occurred running %s") % proc_name
        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, title)
        dlg.format_secondary_text(exc_only_str)

        alignment = gtk.Alignment(0.0, 0.0, 1.0, 1.0)
        alignment.set_padding(0, 0, 12, 12)
        dlg.vbox.pack_start(alignment)
        alignment.show()

        expander = gtk.Expander(_("_More Information"));
        expander.set_use_underline(True)
        expander.set_spacing(6)
        alignment.add(expander)
        expander.show()

        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled.set_size_request(-1, 200)
        expander.add(scrolled)
        scrolled.show()

        label = gtk.Label(exc_str)
        label.set_alignment(0.0, 0.0)
        label.set_padding(6, 6)
        label.set_selectable(True)
        scrolled.add_with_viewport(label)
        label.show()

        def response(widget, id):
            widget.destroy()

        dlg.connect("response", response)
        dlg.set_resizable(True)
        dlg.show()

    # define a mapping of param types to edit objects ...
    class StringEntry(gtk.Entry):
        def __init__(self, default="", **kwargs): # modif C.L.
            gtk.Entry.__init__(self)
            self.set_text(str(default))
            self.set_activates_default(True)
            #~ self.connect('paste-clipboard', lambda _: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            self.get_buffer().connect('inserted-text', lambda _1, _2, _3, _4: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            self.get_buffer().connect('deleted-text', lambda _1, _2, _3: kwargs['dialg'].la_previ.invalidate()) # modif C.L

        def get_value(self):
            return self.get_text()


    class LabelTitle(gtk.Label):
        #def __init__ (self, default="", **kwargs): # modif C.L. ???
        def __init__(self, default="", align = (0.0, 0.0)):		
            gtk.Label.__init__(self)
            self.set_label(str(default))
            self.set_use_markup(True)
            if align != None:
                self.set_alignment(align[0],align[1])
        def get_value(self):
            return self.get_label()

    class TextEntry(gtk.ScrolledWindow):
        def __init__ (self, default="", **kwargs): # modif C.L.
            gtk.ScrolledWindow.__init__(self)
            self.set_shadow_type(gtk.SHADOW_IN)

            self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            self.set_size_request(100, -1)

            self.view = gtk.TextView()
            self.add(self.view)
            self.view.show()
            self.buffer = self.view.get_buffer()
            self.set_value(str(default))
            self.buffer.connect('changed', lambda _: kwargs['dialg'].la_previ.invalidate()) # modif C.L.			

        def set_value(self, text):
            self.buffer.set_text(text)

        def get_value(self):
            return self.buffer.get_text(self.buffer.get_start_iter(), self.buffer.get_end_iter())

    class IntEntry(StringEntry):
        def get_value(self):
            try:
                return int(self.get_text())
            except ValueError, e:
                raise EntryValueError, e.args

    class FloatEntry(StringEntry):
            def get_value(self):
                try:
                    return float(self.get_text())
                except ValueError, e:
                    raise EntryValueError, e.args

    def precision(step):
        # calculate a reasonable precision from a given step size
        if math.fabs(step) >= 1.0 or step == 0.0:
            digits = 0
        else:
            digits = abs(math.floor(math.log10(math.fabs(step))));
        if digits > 20:
            digits = 20
        return int(digits)

    class SliderEntry(gtk.HScale):
        # bounds is (upper, lower, step)
        def __init__(self, default=0, bounds=(0, 100, 5), **kwargs): # modif C.L.
            step = bounds[2]
            self.adj = gtk.Adjustment(default, bounds[0], bounds[1],
                                      step, 10 * step, 0)
            # modif C.L.
            def react(adj):
                return self.get_toplevel().la_previ.invalidate()
            #~ self.adj.connect('value-changed', lambda adj: self.get_toplevel().la_previ.invalidate())
            self.adj.connect('value-changed', react)
            # end modif C.L.
            gtk.HScale.__init__(self, self.adj)
            self.set_digits(precision(step))

        def get_value(self):
            return self.adj.value

    class SpinnerEntry(gtk.SpinButton):
        # bounds is (upper, lower, step)
        def __init__(self, default=10, bounds=(0, 100, 5), **kwargs): # modif C.L.
            step = bounds[2]
            self.adj = gtk.Adjustment(default, bounds[0], bounds[1],
                                      step, 10 * step, 0)
            self.adj.connect('value-changed', lambda adj: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            gtk.SpinButton.__init__(self, self.adj, step, precision(step))


    class ToggleEntry(gtk.ToggleButton):
        #~ def __init__(self, default=0):
        def __init__(self, default=0,labels=[_("No"),_("Yes")], **kwargs): # modif C.L.
            
            gtk.ToggleButton.__init__(self)
            self.labels = labels
            self.label = gtk.Label(labels[0])
            self.add(self.label)
            self.label.show()

            #~ self.connect("toggled", self.changed)
            # modif C.L.
            def react(tog):
                self.changed(tog)
                return kwargs['dialg'].la_previ.invalidate()
            self.connect("toggled", react)
            # end modif C.L.


            self.set_active(default)

        def changed(self, tog):
            if tog.get_active():
                self.label.set_text(self.labels[1])
            else:
                self.label.set_text(self.labels[0])

        def get_value(self):
            return self.get_active()



    class RadioEntry(gtk.VBox):
        def __init__(self, default=0, items=((_("Yes"), 1), (_("No"), 0)), **kwargs): # modif C.L.
            gtk.VBox.__init__(self, homogeneous=False, spacing=2)

            button = None

            for (label, value) in items:
                button = gtk.RadioButton(button, label)
                self.pack_start(button)
                button.show()

                button.connect("toggled", self.changed, value)

                if value == default:
                    button.set_active(True)
                    self.active_value = value

        def changed(self, radio, value):
            if radio.get_active():
                self.active_value = value

        def get_value(self):
            return self.active_value

    class ComboEntry(gtk.ComboBox):
        def __init__(self, default=0, items=(), **kwargs): # modif C.L.
            store = gtk.ListStore(str)
            for item in items:
                store.append([item])

            gtk.ComboBox.__init__(self, model=store)

            cell = gtk.CellRendererText()
            self.pack_start(cell)
            self.set_attributes(cell, text=0)

            self.set_active(default)
            self.connect('changed', lambda adj: kwargs['dialg'].la_previ.invalidate()) # modif C.L.

        def get_value(self):
            return self.get_active()

    def FileSelector(default="", title=None, **kwargs): # modif C.L.
        # FIXME: should this be os.path.separator?  If not, perhaps explain why?
        if default and default.endswith("/"):
            if default == "/": default = ""
            return DirnameSelector(default)
        else:
            return FilenameSelector(default, title=title, save_mode=False)

    class FilenameSelector(gtk.HBox):
        #gimpfu.FileChooserButton
        def __init__(self, default, save_mode=True, title=None, **kwargs): # modif C.L.
            super(FilenameSelector, self).__init__()
            if not title:
                self.title = _("Python-Fu File Selection")
            else:
                self.title = title
            self.save_mode = save_mode
            box = self
            self.entry = gtk.Entry()
            image = gtk.Image()
            image.set_from_stock(gtk.STOCK_FILE, gtk.ICON_SIZE_BUTTON)
            self.button = gtk.Button()
            self.button.set_image(image)
            box.pack_start(self.entry)
            box.pack_start(self.button, expand=False)
            self.button.connect("clicked", self.pick_file)
            if default:
                self.entry.set_text(default)

        def show(self):
            super(FilenameSelector, self).show()
            self.button.show()
            self.entry.show()

        def pick_file(self, widget):
            entry = self.entry
            dialog = gtk.FileChooserDialog(
                         title=self.title,
                         action=(gtk.FILE_CHOOSER_ACTION_SAVE
                                     if self.save_mode else
                                 gtk.FILE_CHOOSER_ACTION_OPEN),
                         buttons=(gtk.STOCK_CANCEL,
                                gtk.RESPONSE_CANCEL,
                                gtk.STOCK_SAVE
                                     if self.save_mode else
                                gtk.STOCK_OPEN,
                                gtk.RESPONSE_OK)
                        )
            dialog.set_alternative_button_order ((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))
            dialog.show_all()
            response = dialog.run()
            if response == gtk.RESPONSE_OK:
                entry.set_text(dialog.get_filename())
            dialog.destroy()

        def get_value(self):
            return self.entry.get_text()


    class DirnameSelector(gtk.FileChooserButton):
        def __init__(self, default="", **kwargs): # modif C.L.
            gtk.FileChooserButton.__init__(self,
                                           _("Python-Fu Folder Selection"))
            self.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
            if default:
                self.set_filename(default)

        def get_value(self):
            return self.get_filename()

    # modif C.L.
    class ColorSelector(gimpui.ColorSelector):
        def __init__(self, default=gimpcolor.RGB(1.0, 0, 0), **kwargs):
            ret = gimpui.ColorSelector.__init__(self, default)
            self.connect('color-changed', lambda _: kwargs['dialg'].la_previ.invalidate())
            return ret
    class FontSelector(gimpui.FontSelector):
        def __init__(self, default="Sans", **kwargs):
            ret = gimpui.FontSelector.__init__(self, default)
            self.connect('font-set', lambda _1, _2, _3: kwargs['dialg'].la_previ.invalidate())
            return ret
    class PatternSelector(gimpui.PatternSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.PatternSelector.__init__(self, default)
            self.connect('pattern-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    class GradientSelector(gimpui.GradientSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.GradientSelector.__init__(self, default)
            self.connect('gradient-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    class PaletteSelector(gimpui.PaletteSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.PaletteSelector.__init__(self, default)
            self.connect('palette-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    # end modif C.L.
    class DrawCanvas(gtk.DrawingArea):
        def __init__(self,default, size=(100,100)):
            gtk.DrawingArea.__init__(self)
            self.set_size_request(size[0],size[1])
        def get_value(self):
            return 0

    _edit_mapping = {
            PF_INT8        : IntEntry,
            PF_INT16       : IntEntry,
            PF_INT32       : IntEntry,
            PF_FLOAT       : FloatEntry,
            PF_STRING      : StringEntry,
            PF_COLOR       : ColorSelector, # modif C.L.
            PF_ITEM        : IntEntry,  # should handle differently ...
            PF_IMAGE       : gimpui.ImageSelector,
            PF_LAYER       : gimpui.LayerSelector,
            PF_CHANNEL     : gimpui.ChannelSelector,
            PF_DRAWABLE    : gimpui.DrawableSelector,
            PF_VECTORS     : gimpui.VectorsSelector,

            PF_TOGGLE      : ToggleEntry,
            PF_SLIDER      : SliderEntry,
            PF_SPINNER     : SpinnerEntry,
            PF_RADIO       : RadioEntry,
            PF_OPTION      : ComboEntry,

            PF_FONT        : FontSelector,
            PF_FILE        : FileSelector,
            PF_FILENAME    : FilenameSelector,
            PF_DIRNAME     : DirnameSelector,
            PF_BRUSH       : gimpui.BrushSelector,
            PF_PATTERN     : PatternSelector, # modif C.L.
            PF_GRADIENT    : GradientSelector, # modif C.L.
            PF_PALETTE     : PaletteSelector, # modif C.L.
            PF_TEXT        : TextEntry,
            PF_TAB         : TextEntry,
            PF_DRAW        : DrawCanvas,
            PF_TITLE       : LabelTitle,
    }
    # JFGARCIA - modified: to pass args to the on_run
    if on_run:
        on_run(start_params+tuple(params))

    dialog = gimpui.Dialog(proc_name, "python-fu", None, 0, None, proc_name, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))

    dialog.set_alternative_button_order((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))

    dialog.set_transient()

    # modif C.L.
    hboxPrincipal = gtk.HBox(spacing=10)
	
    dialog.vbox.pack_start(hboxPrincipal)			        


    hboxPrincipal.show()
    
    dialog.la_previ = gimpui.DrawablePreview(start_params[1])
    dialog.la_previ.set_size_request(550,512)               # New Line MrQ	
    dialog.la_previ.la_img = start_params[0]
    dialog.la_previ.la_layer = start_params[1]
    hboxPrincipal.pack_start(dialog.la_previ)

    dialog.set_default_size(900, 650)  					# New Line MrQ
    dialog.la_previ.show()
    # end modif C.L.



    vbox = gtk.VBox(False, 2)
    vbox.set_border_width(2)
    vbox.set_size_request(500, 0)                           # modif C.L. + MrQ
    #~ dialog.vbox.pack_start(vbox)
    hboxPrincipal.pack_start(vbox) # modif C.L.
    vbox.show()

    if blurb:
        if domain:
            try:
                (domain, locale_dir) = domain
                trans = gettext.translation(domain, locale_dir, fallback=True)
            except ValueError:
                trans = gettext.translation(domain, fallback=True)
            blurb = trans.ugettext(blurb)
        box = gimpui.HintBox(blurb)
        vbox.pack_start(box, expand=False)
        box.show()

    table = gtk.Table(1, 2, False)
    table.set_row_spacings(2)
    table.set_col_spacings(2)
    vbox.pack_start(table, expand=True)
    table.show()

    notebook = gtk.Notebook()
    tables=[]
    for i in range(len(params)):
        if params[i][0]==PF_TAB:
            vals = params[i][4]
            currPage = params[i][3]

            for n in vals:
                tbl = gtk.Table(2, 2, False)
                tbl.set_row_spacings(5)
                tbl.set_col_spacings(5)
                tbl.show()
                tables.append({'table':tbl,'rows':0, 'totCols':0})
                notebook.append_page(tbl, gtk.Label(n))
                notebook.set_current_page(currPage)


    if len(tables)>0:
        table.attach(notebook, 0,6,0,1)
        notebook.show()
    else:
        tables.append({'table':table,'rows':0})
    # modif C.L.
    def ifinvalidated(p_la_previ):
        
        # get dimension
        #===============
        x, y = p_la_previ.get_position()
        w, h = p_la_previ.get_size()
        
        # Effet
        #=========
        # has alpha
        lb_alpha = pdb.gimp_drawable_has_alpha(p_la_previ.la_layer)
        
        # img, layer temporaires
        tempImg = pdb.gimp_image_new(w, h, 0)
        #~ tempLayer = pdb.gimp_layer_new(tempImg, w, h, 0, 'temp', 100, 0)
        tempLayer = pdb.gimp_layer_new(tempImg, w, h, lb_alpha, 'temp', 100, 0)
        pdb.gimp_image_add_layer(tempImg, tempLayer, 0)
        
        # copie orig dans tempo
        rgnPreviIn = p_la_previ.la_layer.get_pixel_rgn(x, y, w, h)
        rgnTemp = tempLayer.get_pixel_rgn(0, 0, w, h)
        rgnTemp[0:w, 0:h] = rgnPreviIn[x:x+w, y:y+h]
        
        # effect
        tempImg.undo_group_start()
        tempImg.disable_undo()
        
        params = []

        try:
            for wid in edit_wids:
                params.append(wid.get_value())
        except EntryValueError:
            warning_dialog(dialog, _("Invalid input for '%s'") % wid.desc)
        else:
            try:
                paramsTot = (tempImg, tempLayer) + tuple(params)
                _set_defaults(proc_name, start_params + tuple(params))
                function(*paramsTot)
                
                res = tempImg.active_layer
                
                if not lb_alpha: pdb.gimp_layer_flatten(res)
                
                tempImg.undo_group_end()
                
                rgnRes = res.get_pixel_rgn(0, 0, w, h)
                
                # mise dans previ
                #================
                p_la_previ.draw_buffer(rgnRes[0:w, 0:h], int(3+lb_alpha)*w)
                
                pdb.gimp_image_delete(tempImg)
                
                #pdb.gimp_display_new(tempImg)
            except CancelError:
                pass
            except Exception as e:
                dialog.set_response_sensitive(gtk.RESPONSE_CANCEL, True)
                error_dialog(dialog, proc_name)
                raise
    # end modif C.L.
    def response(dlg, id):
        if id == gtk.RESPONSE_OK:
            dlg.set_response_sensitive(gtk.RESPONSE_OK, False)
            dlg.set_response_sensitive(gtk.RESPONSE_CANCEL, False)
            params = []
            try:
                for wid in edit_wids:
                    params.append(wid.get_value())
            except EntryValueError:
                warning_dialog(dialog, _("Invalid input for '%s'") % wid.desc)
            else:
                try:
                    dialog.res = run_script(params)
                except Exception:
                    dlg.set_response_sensitive(gtk.RESPONSE_CANCEL, True)
                    error_dialog(dialog, proc_name)
                    raise
        gtk.main_quit()
    dialog.la_previ.connect('invalidated', ifinvalidated) # modif C.L.
    dialog.connect("response", response)
    iniX=0
    row=0
    edit_wids = []

    for i in range(len(params)):
        paramsi = params[i]
        pf_type = paramsi[0]
        name, desc = (paramsi[1], paramsi[2])
        def_val = defaults[i]

        tab, incr, rows, connect, cols = (0, 1, 1, None, 1)
        if len(params[i])>5:
            xtra = params[i][5]
            incr = int(getXtra(xtra, 'newLine', 1))
            if incr==0 : cols = 1
            cols = getXtra(xtra, 'cols'   , cols)
            iniX = getXtra(xtra, 'iniCol' , iniX)
            rows = getXtra(xtra, 'rows'   , 1)
            tab  = getXtra(xtra, 'tab'    , 0)
            connect = getXtra(xtra,'connect', None)
        else:
            iniX = 0
        tbl = tables[tab]['table']
        row = tables[tab]['rows']


        if pf_type==PF_TAB:
            wid = _edit_mapping[pf_type](None)
        else:
            if pf_type != PF_TITLE:
                label = gtk.Label(desc)
                label.set_use_underline(True)
                label.set_alignment(0.0, 0.5)
                tbl.attach(label, iniX + 1, iniX + 2, row, row + rows, xoptions = gtk.FILL)
                label.show()

            if pf_type in (PF_SPINNER, PF_SLIDER, PF_RADIO, PF_OPTION, PF_DRAW):
                wid = _edit_mapping[pf_type](def_val, params[i][4], dialg=dialog) # modif C.L.
            elif pf_type==PF_TOGGLE:
                if isinstance(paramsi[4], list):
                    wid = _edit_mapping[pf_type](def_val, paramsi[4],dialg=dialog)
                else:
                    wid = _edit_mapping[pf_type](def_val)				


            elif pf_type==PF_TITLE:
                wid = _edit_mapping[pf_type](desc)
            else:
                wid = _edit_mapping[pf_type](def_val, dialg=dialog) # modif C.L.
            if pf_type!=PF_TITLE:
                label.set_mnemonic_widget(wid)
                tbl.attach(wid, iniX + 2, iniX + 2 + cols, row, row + rows, yoptions=0)
                iniX = iniX + cols + 1
            else:
                tbl.attach(wid, iniX, iniX+cols, row, row + rows, xoptions=gtk.FILL)
            # jfGarcia: strip html tags from toolTip
            tTip = re.sub('<[^<]+?>', '', desc)
            if pf_type != PF_TEXT:
                wid.set_tooltip_text(tTip)
            else:
                # Attach tip to TextView, not to ScrolledWindow
                wid.view.set_tooltip_text(tTip)

            wid.set_name(name)
            if connect:
                try:
                    wid.connect(connect['signal'],connect['function'])
                except Exception, e:
                    pass
            wid.show()
            wid.desc = desc

        edit_wids.append(wid)

        tables[tab]['rows'] += incr
        if incr==1:
           iniX=0

    progress_vbox = gtk.VBox(False, 6)
    vbox.pack_end(progress_vbox, expand=False)
    progress_vbox.show()

    progress = gimpui.ProgressBar()
    progress_vbox.pack_start(progress)
    progress.show()
	
    dialog.show()
    gtk.main()

    if hasattr(dialog, "res"):
        res = dialog.res
        dialog.destroy()
        return res
    else:
        dialog.destroy()
        raise CancelError

def _run(proc_name, params):
    run_mode = params[0]
    func = _registered_plugins_[proc_name][10]

    if run_mode == RUN_NONINTERACTIVE:
        return apply(func, params[1:])

    script_params = _registered_plugins_[proc_name][8]

    min_args = 0
    if len(params) > 1:
        for i in range(1, len(params)):
            param_type = _obj_mapping[script_params[i - 1][0]]
            if not isinstance(params[i], param_type):
                break

        min_args = i

    if len(script_params) > min_args:
        start_params = params[:min_args + 1]

        if run_mode == RUN_WITH_LAST_VALS:
            default_params = _get_defaults(proc_name)
            params = start_params + default_params[min_args:]
        else:
            params = start_params
    else:
       run_mode = RUN_NONINTERACTIVE

    if run_mode == RUN_INTERACTIVE:
        try:
            res = _interact(proc_name, params[1:])
        except CancelError:
            return
    else:
        res = apply(func, params[1:])

    gimp.displays_flush()

    return res

def main():
    """This should be called after registering the plug-in."""
    gimp.main(None, None, _query, _run)

def fail(msg):
    """Display an error message and quit"""
    gimp.message(msg)
    raise error, msg

def N_(message):
    return message
	
	
#####################################################################################################################################################
############################################# END fu_preview 0.1.1 by Claude Lion $ J.F. Garcia #####################################################
#####################################################################################################################################################
import os, string, sys
import decimal
import gettext
import unicodedata
gettext.install("gimp20", gimp.locale_directory, unicode=True)
import math, random, os, sys



###############################################################################################################################
############################################ START dial-marks-0.2.py by Ofnuts ################################################
###############################################################################################################################

# GIMP plugin to generate a path for dial marks
# (c) Ofnuts 2014
#
#   History:
#   v0.0: 2013-11-15: First published version
#   v0.1: 2013-11-15: Add offset/start angle
#   v0.2: 2014-10-22: Add straight/round sides

def point(origin,radius,angle):
	originX,originY=origin
	x=radius*math.cos(angle)+originX
	y=radius*math.sin(angle)+originY
	return (x,y)

def tangentDistanceForArc(radius,halfAngle):
	return radius * (4./3.) * (1-math.cos(halfAngle))/math.sin(halfAngle)

def markPointsSquare(center,innerRadius,outerRadius,angle,width):
	p1=point(center,innerRadius,angle-width)
	p2=point(center,outerRadius,angle-width)
	p3=point(center,outerRadius,angle+width)
	p4=point(center,innerRadius,angle+width)
	return [p1]*3+[p2]*3+[p3]*3+[p4]*3

def markPointsRound(center,innerRadius,outerRadius,angle,width):
	innerTg=tangentDistanceForArc(innerRadius,width)
	outerTg=tangentDistanceForArc(outerRadius,width)
	p1=point(center,innerRadius,angle-width)
	p2=point(center,outerRadius,angle-width)
	p3=point(center,outerRadius,angle+width)
	p4=point(center,innerRadius,angle+width)
	t1=point(p1,innerTg,angle-width+math.pi/2)
	t2=point(p2,outerTg,angle-width+math.pi/2)
	t3=point(p3,outerTg,angle+width-math.pi/2)
	t4=point(p4,innerTg,angle+width-math.pi/2)
	return [t1,p1,p1,p2,p2,t2,t3,p3,p3,p4,p4,t4] 

def flattenToStroke(points):
	'''
	Takes a list of points as tuples and flattens it to a list of coordinates 
	'''
	return [coord for point in points for coord in point]

def dialMarks_cc(image,centerX,centerY,innerRadius,outerRadius,marksCount,marksWidth,startAngle,rounded):
	
	if marksWidth>90.:
		pdb.gimp_message('Maximum mark width is 90°')
		return
	
	angleStep=2*math.pi/marksCount
	width=marksWidth*math.pi/360 # Actually half-width
	offset=startAngle*math.pi/180
	
	markPoints=[markPointsSquare,markPointsRound][rounded]
	#marksPath=pdb.gimp_vectors_new(image,'Marks: %d, %2.1f, %3.1f, %3.1f' % (marksCount,marksWidth,innerRadius,outerRadius))	# by Orginal	
	marksPath=pdb.gimp_vectors_new(image,'Marks: %d, %d' % (outerRadius-innerRadius,marksCount))	                            # by MrQ outerRadius-innerRadius=rim_thickness
	for i in range(marksCount):
		angle=offset+angleStep*i
		points=flattenToStroke(markPoints((centerX,centerY),innerRadius,outerRadius,angle,width))
		sid = pdb.gimp_vectors_stroke_new_from_points(marksPath,0, len(points),points, True)
	pdb.gimp_image_add_vectors(image,marksPath,0)
	marksPath.visible=False
##########################################################################################################################################
################################## END dial-marks-0.2.py by Ofnuts #######################################################################
##########################################################################################################################################
	
###############################################################################################################
#################### Start text-along-path-04 by Ofnuts #######################################################
###############################################################################################################
v,r,l=gimp.version
use28=(v==2) and (r>6)

debug=False
def trace(s):
	if debug:
		print s

BASELINE=0 	# The baseline
TOP=1		# The top of the character box	
BOTTOM=2	# The bottom of the character box
MIDDLEBOX=3	# The middle of the character box
TOPCAPS=4	# The top of uppercase characters
MIDDLECAPS=5	# The middle of uppercase characters

# Characters that won't produce a path (space, etc...)
blankCharacters=' '

TEXT_JUSTIFY_LEFT=0
TEXT_JUSTIFY_RIGHT=1
TEXT_JUSTIFY_CENTER=2
TEXT_JUSTIFY_FILL=3

#----------------------------------------------------
# Enhanced path Stroke
#----------------------------------------------------
class DirectionStroke:
			
	def __init__(self,stroke):
		self.stroke=stroke
		points,closed=self.stroke.points
		bezierCurves=(len(points)/3)-1

		self.precision=.05/bezierCurves

		self.length=stroke.get_length(self.precision)

	def computeThetaWithSlope(self,fromPoint,toPoint,slope):
		dX=toPoint[0]-fromPoint[0]
		dY=toPoint[1]-fromPoint[1]
		
		if math.fabs(dX) < .001:
			theta=math.copysign(math.pi/2,dY)
		else:
			theta=math.atan(slope)
			if dX<0:
				theta=theta+math.pi
		return theta % (math.pi*2)
		
	# Facade method, returns only if point is OK
	def getRawPointAtDist(self,dist):
		x,y,slope,valid=self.stroke.get_point_at_dist(dist,self.precision)
		if not valid: # Normally quite unlikely, since we avoid the cases where this could happen
			raise Exception('No point found at %6.4f in stroke %s with length %6.4f' % (dist,self.stroke.ID,self.length))
		return x,y,slope

	def getPointAtDist(self,dist):
		# Now process regular case
		x,y,slope=self.getRawPointAtDist(dist)

		delta=.5
		toX,toY,s=self.getRawPointAtDist(dist+delta) 
		theta=self.computeThetaWithSlope([x,y],[toX,toY],slope)

		return (x,y,theta)

#----------------------------------------------------
# Character in text
#----------------------------------------------------
class Character(object):
	def __init__(self,character,width,height):
		self.character=character
		self.width=width
		self.height=height
		self.path=None
		self.kerning=0
		self.position=0
		self.first=False
		self.last=False
		
	def __str__(self):
		if self.path:
			return "<'%s' (%d,%d) @%3.2f, %d stroke(s)>" % (self.character,self.width,self.height,self.position,len(self.path.strokes))
		else:
			return "<'%s' (%d,%d) @%3.2f, (no path)>" % (self.character,self.width,self.height,self.position)
			
#----------------------------------------------------
# Text to work on
#----------------------------------------------------
class Text(object):
	
	def __init__(self,text,fontName,fontSize,
			layout=TEXT_JUSTIFY_CENTER, pivotYChoice=BASELINE, 
			useKerning=True, extraSpacing=0, verticalAdjust=0,
			keepPaths=False, keepUpright=False,
			wiggleXPercent=0, wiggleYPercent=0, wiggleTheta=0):
		self.text=unicode(text,'utf-8','strict')
		self.fontName=fontName
		self.fontSize=fontSize
		self.pivotYChoice=pivotYChoice
		self.useKerning=useKerning
		self.extraSpacing=extraSpacing
		self.verticalAdjust=verticalAdjust
		self.keepUpright=keepUpright
		self.layout=layout
		self.keepPaths=keepPaths
		self.wiggleXPercent=wiggleXPercent
		self.wiggleYPercent=wiggleYPercent
		self.wiggleTheta=wiggleTheta
		
		self.pivotY=None
		self.characters=[]
		self.workImage=gimp.Image(int(fontSize*4),int(fontSize*4), RGB)
		self.workImage.disable_undo()
		
	def __del__(self):
		gimp.delete(self.workImage)

	def extents(self,text):
		return pdb.gimp_text_get_extents_fontname(text, self.fontSize, PIXELS, self.fontName)
		
	def textPath(self,text):
		if len(text) == 1 and text in blankCharacters:
			path=None
		else:
			l=pdb.gimp_text_fontname(self.workImage, None, 0, 0, text, 0, True, self.fontSize, PIXELS, self.fontName)
			path=pdb.gimp_vectors_new_from_text_layer(self.workImage,l)
			self.workImage.remove_layer(l)
		return path
		
	def initializeCharacter(self,c):
		width,height,ascent,descent=self.extents(c)
		addedChar=Character(c,width,height)
		addedChar.path=self.textPath(c)
		# compute kerning unless first character
		if self.useKerning and len(self.characters) > 0:
			rightChar=self.characters[-1]
			kw,kh,ka,kd=self.extents(rightChar.character+addedChar.character)
			addedChar.kerning=kw-(addedChar.width+rightChar.width)
			if addedChar.kerning !=0:
				trace('Kerning %c -> %c: %3.2f' % (rightChar.character,addedChar.character,addedChar.kerning))

		self.characters.append(addedChar)
	
	def initializeCharacters(self):
		trace('Text: %s' % self.text)
		for c in self.text:
			self.initializeCharacter(c)
			
		self.characters[0].first=True
		self.characters[-1].last=True
		lastChar=self.characters[-1]
			
	def computePivotY(self):
		sampleText='X'
		# obtain path for sample text
		path=self.textPath(sampleText)
		
		width,height,ascent,descent=self.extents(sampleText)
		
		# Gather all Y values (anchors and handles) in all strokes for the bounding box 
		allY=[stroke.points[0][1::2] for stroke in path.strokes] # List of lists
		allY=[y for stroke in allY for y in stroke] # flattened
		minY=min(allY)
		maxY=max(allY)
		
		self.wiggleYMax=height
		topCaps=minY+(maxY-ascent)
		# compute all possible pivotY and keep the good one (easier to debug)
		pivot=[0,0,0,0,0,0]
		pivot[BASELINE]=ascent
		pivot[TOP]=0
		pivot[BOTTOM]=height
		pivot[MIDDLEBOX]=height/2.
		pivot[MIDDLECAPS]=(topCaps+ascent)/2.
		pivot[TOPCAPS]=topCaps
		#trace(pivot)
		self.pivotY=pivot[self.pivotYChoice]+self.verticalAdjust
		
	# Lay out the characters on the stroke, by computing their.position coordinate
	def layoutOnStroke(self,stroke):
		# Plain text width
		plainTextWidth=0
		for c in self.characters:
			plainTextWidth=plainTextWidth+c.width+c.kerning
		
		# Check fit on stroke
		if self.layout != TEXT_JUSTIFY_FILL:
			# Check with requested extra spacing if not justified
			textWidth=plainTextWidth+self.extraSpacing*(len(self.characters)-1)
			if textWidth > stroke.length:
				raise Exception('Text width (%3.2f) larger than path stroke length (%3.2f)' % (textWidth,stroke.length))
			extraSpacing=self.extraSpacing
		else: # TEXT_JUSTIFY_FILL
			extraSpacing=(stroke.length-plainTextWidth)/(len(self.characters)-1)
			textWidth=stroke.length
		self.wiggleXMax=textWidth/len(self.characters)
		# Compute offset of left character
		if self.layout==TEXT_JUSTIFY_LEFT:
			offset=0
		elif self.layout==TEXT_JUSTIFY_RIGHT:
			offset=stroke.length-textWidth
		elif self.layout==TEXT_JUSTIFY_CENTER:
			offset=(stroke.length-textWidth)/2.
		else: #TEXT_JUSTIFY_FILL, normally
			offset=0
		
		trace('Plain text width: %3.2f, stroke length: %3.2f, start offset: %3.2f, extraSpacing: %3.2f, actual text width: %3.2f ' % (plainTextWidth,stroke.length,offset, extraSpacing, textWidth))
		
		# set.position for each character
		for c in self.characters:
			offset=offset+c.width/2.+c.kerning
			c.position=offset
			offset=offset+c.width/2.+extraSpacing
			
	# compute final pos for character
	def computeFinalPos(self,c,stroke):
		x,y,slope=stroke.getPointAtDist(c.position)
		if self.keepUpright:
			tilt=0
		else:
			tilt=slope*180/math.pi
		
		wiggleXRange=self.wiggleXMax*self.wiggleXPercent/100.
		wiggleYRange=self.wiggleYMax*self.wiggleYPercent/100.

		wx=random.uniform(-wiggleXRange,wiggleXRange)
		wy=random.uniform(-wiggleYRange,wiggleYRange)
		wtilt=random.uniform(-self.wiggleTheta,self.wiggleTheta)

		return x+wx*math.cos(slope)-wy*math.sin(slope),y+wy*math.cos(slope)+wx*math.sin(slope),tilt+wtilt
		
	# copy path (possibly to different image)
	def copyPath(self,sourcePath,name,img):
		newPath=pdb.gimp_vectors_new(img,name)
		pdb.gimp_image_add_vectors(img,newPath,0)
		self.copyStrokes(sourcePath,newPath)
		return newPath
		
	def copyStrokes(self,sourcePath,targetPath):
		for s in sourcePath.strokes:
			points,closed=s.points
			sid = pdb.gimp_vectors_stroke_new_from_points(targetPath,0, len(points),points, closed)
		
		
	def moveCharacterToStroke(self,c,stroke,img):
		if not c.path:
			return # nothing to do on blank characters
		# copy character path (defined on work image) to final image
		path=self.copyPath(c.path,"'"+c.character+"'",img)
		path.visible=False;
		x,y,tilt=self.computeFinalPos(c,stroke)
		trace('"%s": actual: %3.2f' % (c,x))
		for s in path.strokes:
			 s.rotate(c.width/2.,self.pivotY,tilt)
			 s.translate(x-c.width/2.,y-self.pivotY)
		self.copyStrokes(path,self.finalPath)
		if not self.keepPaths:
			pdb.gimp_image_remove_vectors(img,path)
			 
	def moveCharactersToStroke(self,stroke,img,pathName):
		self.finalPath=pdb.gimp_vectors_new(img,"'%s' over <%s>" % (self.text,pathName))
		self.finalPath.visible=True
		pdb.gimp_image_add_vectors(img,self.finalPath,0)

		trace("Max wiggle X,Y: %3.2f,%3.2f" % (self.wiggleXMax,self.wiggleYMax))
		for c in self.characters:
			self.moveCharacterToStroke(c,stroke,img)
			
	def run(self,path,image):
		if not len(path.strokes):
			raise Exception('No strokes in path "%s"' % path.name)
		self.computePivotY()
		self.initializeCharacters()
		stroke=DirectionStroke(path.strokes[0])
		self.layoutOnStroke(stroke)
		self.moveCharactersToStroke(stroke,image,path.name)
		pdb.gimp_displays_flush()	

def text_along_path(image,layer,path,text,fontName,fontSize,
			layout,pivotYChoice,
			useKerning,extraSpacing,verticalAdjust,
			keepPaths, keepUpright,
			wiggleXPercent,wiggleYPercent,wiggleTheta):
				
	pdb.gimp_image_undo_group_start(image)
	try:
		# extract data from text layer if provided
		if layer:
			trace('--1--')
			if use28:
				isText=pdb.gimp_drawable_is_text_layer(layer.ID)
			else:
			        isText=pdb.gimp_drawable_is_text_layer(layer)
			if not isText:
				raise Exception('Drawable "%s" is not a text layer' % layer.name)
			trace('--2--')
			text = pdb.gimp_text_layer_get_text(layer)
			if not text:
				text='#TEXT#'
				# raise Exception('No text in layer "%s"' % layer.name)
			trace('--3--')
			fontName = pdb.gimp_text_layer_get_font(layer)
			trace('--4--')
			fontSize, unit = pdb.gimp_text_layer_get_font_size(layer)
			trace("Font: %d, %s" % (fontSize,unit))
			trace('--5--')
			layout = pdb.gimp_text_layer_get_justification(layer)
			trace('--6--')
			useKerning = pdb.gimp_text_layer_get_kerning(layer)
			trace('--7--')
			extraSpacing = pdb.gimp_text_layer_get_letter_spacing(layer)
			trace('--8--')
			
		if not fontName:
			fontName = pdb.gimp_context_get_font()
		if not text:
			raise Exception('No text provided')
		text=Text(text,fontName,fontSize,
				layout,pivotYChoice,
				useKerning, extraSpacing, verticalAdjust,
				keepPaths, keepUpright,
				wiggleXPercent,wiggleYPercent,wiggleTheta)
		text.run(path,image)
	except Exception as e:
		print e
		print dir(e)
		print e.args
		trace(e.args[0])
		pdb.gimp_message(e.args[0])
	pdb.gimp_image_undo_group_end(image)


# Call form path list: no text layer provided, use full options
def text_along_path_full_coin(image,path,text,fontName,fontSize,
			layout,pivotYChoice,
			useKerning,extraSpacing,verticalAdjust,
			keepPaths, keepUpright,
			wiggleXPercent,wiggleYPercent,wiggleTheta):
	text_along_path(image,None,path,text,fontName,fontSize,
			layout,pivotYChoice,
			useKerning,extraSpacing,verticalAdjust,
			keepPaths, keepUpright,
			wiggleXPercent,wiggleYPercent,wiggleTheta)
	
# Call from Layers list: many options taken from text layer
def text_along_path_layer(image,layer,path,
			pivotYChoice,verticalAdjust,
			keepPaths, keepUpright,
			wiggleXPercent,wiggleYPercent,wiggleTheta):
	text_along_path(image,layer,path,None,None,None,
			None,pivotYChoice,
			None,None,verticalAdjust,
			keepPaths, keepUpright,
			wiggleXPercent,wiggleYPercent,wiggleTheta)
##############################################################################################################			
############################## End text-along-path-04 by Ofnuts ##############################################
##############################################################################################################			


####################################################################################################################
################# START gegl_command by Kevin Payne & Claude Lion ##################################################
####################################################################################################################

from sys import platform
from ctypes import *

def load_library (library_name):
	if platform == "linux" or platform == "linux2":
		library_name = library_name + '.so.0'
	elif platform == "win32":
		from ctypes.util import find_library
		library_name = find_library (library_name + "-0")
	else:
		raise BaseException ("TODO")
	return CDLL (library_name)

gimpC = load_library ('libgimp-2.0')


def gegl_batch(image, drawable, gegl_graph_string):
	
	class GeglBuffer(Structure):
		pass
	drawable_id = drawable.ID
	
	gegl = load_library ('libgegl-0.4')
	gegl.gegl_init (None, None)
	gimpC.gimp_drawable_get_shadow_buffer.restype = POINTER (GeglBuffer)
	gimpC.gimp_drawable_get_buffer.restype        = POINTER (GeglBuffer)

	x = c_int (); y = c_int (); w = c_int (); h = c_int ()
	non_empty,x,y,w,h = pdb.gimp_drawable_mask_intersect (drawable)
	args = [b"string", c_char_p (gegl_graph_string), c_void_p ()]
	
	if non_empty:
		source = gimpC.gimp_drawable_get_buffer (drawable_id)
		target = gimpC.gimp_drawable_get_shadow_buffer (drawable_id)
		gegl.gegl_render_op (source, target, "gegl:gegl", *args)
		gegl.gegl_buffer_flush (target)
		gimpC.gimp_drawable_merge_shadow (drawable_id, PushUndo = True)
		gimpC.gimp_drawable_update (drawable_id, x, y, w, h)
		gimpC.gimp_displays_flush ()
		
####################################################################################################################
################# END gegl_command #################################################################################
####################################################################################################################

def mura_cc(img,layer,w,h,many):
	rotate_angle=0
	
	group_new = pdb.gimp_layer_group_new(img)
	parent = pdb.gimp_item_get_parent(group_new)
	position = pdb.gimp_image_get_item_position(img, layer)
	pdb.gimp_image_insert_layer(img,group_new, None, 0) 
	group_new.name = "Many Stars"
		
	for item in range(0,int(many)):
		
		inc_angle=360/many
		rotate_angle=rotate_angle + inc_angle
		pdb.gimp_image_get_active_layer(img)
		copy = pdb.gimp_layer_copy(layer, TRUE)
		pdb.gimp_image_insert_layer(img, copy, None, -1)
		pdb.gimp_item_transform_rotate(copy, rotate_angle*0.01745329252 ,TRUE, w/2, h/2)			
		pdb.gimp_image_reorder_item(img, copy, group_new, position)
		pdb.gimp_layer_resize_to_image_size(copy)
	pdb.gimp_image_merge_layer_group(img, group_new)
	pdb.gimp_image_resize_to_layers(img)
	
#####################################################################################################################
#####################################################################################################################
	
def MrQ_Coin_Creator(image,layer,diameter,margin,fg_1,fg_2,rim_thickness,edg,marksCount,marksWidth,
	font_t,fs_t,ex_t,text_t, 
	ct,font_c,fs_c,text_c, line_sp,
	font_b,fs_b,ex_b,type_b,text_b,
	make_r,fg_ring,out_rad,in_rad,make_st,fg_stars,size_star,radius_s,many_star,
	code_c,bh_c,legend_c,code_t,bh_3,legend_iv,code_r,bh_r,invert_r,code_s,bhs,invert_s,
	help, ptab):


	pdb.gimp_selection_none(image)
	
	pdb.gimp_image_set_active_layer(image,layer)
	for layer in image.layers:
		pdb.gimp_image_set_active_layer(image,layer)
		pdb.gimp_item_set_visible(layer, 0)
		
	layer_copy = pdb.gimp_layer_new_from_drawable(layer, image)
	pdb.gimp_image_insert_layer(image, layer_copy, None, 0)
	layer_copy.name="Copy Orginal Layer"


	drawable = gimp.Layer(image, "Coin", int(diameter+2*margin), int(diameter+2*margin), RGBA_IMAGE, 100, 0)
	image.add_layer(drawable, 1)

	if margin>0: # " CENTERED "
		wL = pdb.gimp_drawable_width(drawable)
		hL = pdb.gimp_drawable_height(drawable)		
		
		wI = pdb.gimp_image_width(image)
		hI = pdb.gimp_image_height(image)

		sw=(wI-wL)/2
		sh=(hI-hL)/2		
		pdb.gimp_layer_set_offsets(drawable, sw, sh)

	pdb.gimp_image_resize_to_layers(image)			
	if ct==2:
		pdb.gimp_image_set_active_layer(image,layer)	
		pdb.gimp_layer_scale(layer, (diameter*0.6), (diameter*0.6), 1)
			
		wL = pdb.gimp_drawable_width(layer)
		hL = pdb.gimp_drawable_height(layer)
		wI = pdb.gimp_drawable_width(drawable)
		hI = pdb.gimp_drawable_height(drawable)		

		sw=(wI-wL)/2
		sh=(hI-hL)/2		
		pdb.gimp_layer_set_offsets(layer, sw, sh)
		pdb.gimp_layer_resize_to_image_size(layer)
		pdb.gimp_displays_flush()         
	wI = pdb.gimp_drawable_width(layer)
	hI = pdb.gimp_drawable_height(layer)
	pdb.gimp_image_select_ellipse(image,0,margin+diameter*0.2,margin+diameter*0.2, diameter*0.6, diameter*0.6)
	pdb.gimp_selection_invert(image)
	if ct==1 or ct==2 or ct==3:
		pdb.gimp_edit_clear(layer)
	#pdb.gimp_edit_cut(layer)
	pdb.gimp_selection_none(image)
	
	white='#ffffff'
	gray ='#ffffff'
	black='#000000'

	####### coin layer
	pdb.gimp_image_select_ellipse(image,0,0+margin,0+margin, diameter, diameter)
	layer_copy = pdb.gimp_layer_new_from_drawable(drawable, image)
	pdb.gimp_image_insert_layer(image, layer_copy, None, 0)
	coinblank = image.new_layer()
	coinblank.name='coin blank'

	pdb.gimp_context_set_foreground(fg_1)
	pdb.gimp_edit_fill(coinblank, 0)
	coin = pdb.gimp_layer_new_from_visible(image,image,'Coin')
	image.add_layer(coin,0)
	pdb.gimp_item_set_visible(coinblank, False)
	pdb.gimp_item_set_visible(coin, False)
    
####### rim layer

	pdb.gimp_image_select_item(image,0,coin)    
	pdb.gimp_context_set_foreground(fg_2)
	rim = image.new_layer()
	rim.name = 'rim'
	pdb.gimp_edit_fill(rim, 0)
	pdb.gimp_selection_shrink(image, rim_thickness+2) #for a total of 25, less the 7 from the edge vector
	pdb.gimp_edit_cut(rim)
	pdb.gimp_selection_none(image)
	rim = pdb.gimp_image_get_active_layer(image) # layer_fx changes input layer id - get new id with same name.
	pdb.gimp_item_set_visible(rim, False)

####### grooves layer rim_thickness 

	if edg:
		pdb.gimp_context_set_foreground(fg_2)
        #dialMarks_cc(image,centerX,centerY,innerRadius,outerRadius,marksCount,marksWidth,startAngle,rounded)
		dialMarks_cc(image,int(margin+diameter/2),int(margin+diameter/2),float(diameter/2 - rim_thickness),float(diameter/2),int(marksCount),float(marksWidth),1,1)
		marksPath=image.vectors[0]
		pdb.gimp_image_select_item(image, 2, marksPath)
		pdb.gimp_edit_clear(rim)
		pdb.gimp_selection_none(image)		
		type_g='bumpmap'
		gegl_batch(image, rim,'emboss type=bumpmap azimuth=275 elevation=30 depth=50') # % (type_g='bumpmap', azimuth_g, elevation_g, depth_g=50))

####### legend layer

	legend=image.new_layer()
	legend.name = 'Legend'
	pdb.gimp_item_set_visible(legend, False)   

	r = 0.55191502449
	r1 = 1-r  # there is no exact representation of the circle using Bezier curves.

	if type_b==0: # botom L -> R
		b_path = pdb.gimp_vectors_new(image,"b_path")
		pdb.gimp_image_insert_vectors(image, b_path, None, -1)

		x1=margin+diameter/10
		y1=margin+diameter/2
		x2=margin+diameter/2
		y2=margin+diameter*0.9
		x3=margin+diameter*0.9
		y3=margin+diameter/2

		control1_1 = [x1,y1]                           #first control of point 1
		point1 = [x1,y1]                               #point 1
		control1_2 = [x1,max(y1,y2) - abs(y2-y1)*r1]   #second control of point 1

		control2_1 = [max(x1,x2) - abs(x2-x1)*r,y2]    #first control of point 2
		point2 = [x2,y2]                               #point 2
		control2_2 = [abs(x2-x1)*r+max(x1,x2),y2]      #second control of point 2

		control3_2 = [x3,y3]                           #first control of point 3
		point3 = [x3,y3]                               #point 3
		control3_1 = [x3,max(y2,y3) - abs(y3-y2)*r1]   #second control of point 3

		all13points = control1_1 + point1 + control1_2 + control2_1 + point2 + control2_2 + control3_1 + point3 + control3_2;
		pdb.gimp_vectors_stroke_new_from_points(b_path ,0,18,all13points,0)
	   
	if type_b==1: # botom R -> L
		b_path = pdb.gimp_vectors_new(image,"b_path")
		pdb.gimp_image_insert_vectors(image, b_path, None, -1)

		x1=margin+diameter*0.9
		y1=margin+diameter/2
		x2=margin+diameter/2
		y2=margin+diameter*0.9
		x3=margin+diameter/10
		y3=margin+diameter/2

		control1_1 = [x1,y1]                          #first control of point 1
		point1 = [x1,y1]                              #point 1
		control1_2 = [x1,max(y1,y2) - abs(y2-y1)*r1]  #second control of point 1

		control2_1 = [max(x1,x2) - abs(x2-x1)*r1,y2]  #first control of point 2
		point2 = [x2,y2]                              #point 2
		control2_2 = [min(x1,x2) - abs(x2-x1)*r,y2]   #second control of point 2

		control3_2 = [x3,y3]                          #first control of point 3
		point3 = [x3,y3]                              #point 3
		control3_1 = [x3,max(y2,y3) - abs(y3-y2)*r1]  #second control of point 3

		all13points = control1_1 + point1 + control1_2 + control2_1 + point2 + control2_2 + control3_1 + point3 + control3_2;
		pdb.gimp_vectors_stroke_new_from_points(b_path ,0,18,all13points,0)

	if type_b==2:
		a_path = pdb.gimp_vectors_new(image,"a_path")
		pdb.gimp_image_insert_vectors(image, a_path, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(a_path, margin+diameter/2, margin+diameter/2, diameter*0.4, diameter*0.4, 0)

	if type_b!=2:	
		a_path = pdb.gimp_vectors_new(image,"a_path")
		pdb.gimp_image_insert_vectors(image, a_path, None, -1)
	   
		x1=margin+diameter/10
		y1=margin+diameter/2
		x2=margin+diameter/2
		y2=margin+diameter/10
		x3=margin+diameter*0.9
		y3=margin+diameter/2

		control1_1 = [x1,y1]                        #first control of point 1
		point1 = [x1,y1]                            #point 1
		control1_2 = [x1,max(y1,y2) - abs(y2-y1)*r] #second control of point 1

		control2_1 = [max(x1,x2) - abs(x2-x1)*r,y2] #first control of point 2
		point2 = [x2,y2]                            #point 2
		control2_2 = [abs(x2-x1)*r+max(x1,x2),y2]   #second control of point 2

		control3_2 = [x3,y3]                        #first control of point 3
		point3 = [x3,y3]                            #point 3
		control3_1 = [x3,max(y2,y3) - abs(y3-y2)*r] #second control of point 3 

		all13points = control1_1 + point1 + control1_2 + control2_1 + point2 + control2_2 + control3_1 + point3 + control3_2;
		pdb.gimp_vectors_stroke_new_from_points(a_path ,0,18,all13points,0);	
	
####### running ofnuts text along path script 
	#t_text=text_t.encode("utf-8")
	#b_text=text_b.encode("utf-8")
	#c_text=text_c.encode("utf-8")
	# pdb.python_fu_text_along_path_full(image, path, text, fontName, fontSize, layout, pivotY, useKerning, extraSpacing, and 6 more 0,0,0,0,0,0)
	text_along_path_full_coin(image,a_path,text_t,font_t,fs_t,2,5,1,ex_t,0,0,0,0,0,0)
	a_stroked = pdb.gimp_image_get_active_vectors(image)
	if type_b!=2:	
		text_along_path_full_coin(image,b_path,text_b,font_b,fs_b,2,5,1,ex_b,0,0,0,0,0,0)
	b_stroked = pdb.gimp_image_get_active_vectors(image)
	pdb.gimp_selection_none(image)
    
####### set selections for fill and bump
# Deprecated: pdb.gimp_vectors_to_selection(vectors, operation, antialias, feather, feather_radius_x, feather_radius_y)        
# Use:        pdb.gimp_image_select_item(image, operation[CHANNEL-OP-ADD (0), CHANNEL-OP-SUBTRACT (1), CHANNEL-OP-REPLACE (2), CHANNEL-OP-INTERSECT (3)], item)  

	#pdb.gimp_vectors_to_selection(a_stroked,0,1,0,0,0)
	pdb.gimp_image_select_item(image,0,a_stroked)
	#pdb.gimp_vectors_to_selection(b_stroked,0,1,0,0,0)
	pdb.gimp_image_select_item(image,0,b_stroked)
	pdb.gimp_image_set_active_layer(image, legend)
	#pdb.gimp_context_set_foreground(gray) #med-light gray for better bump.
	pdb.gimp_context_set_foreground((fg_1))
	pdb.gimp_edit_fill(legend,0)
	pdb.gimp_selection_none(image)
	#pdb.plug_in_gauss_rle2(image,legend,2,2) #medium blur for less alias on bump.
	gegl_batch(image,legend,'gaussian-blur std-dev-x=1 std-dev-y=1')
	pdb.gimp_vectors_set_visible(a_stroked,0)
	pdb.gimp_vectors_set_visible(b_stroked,0)

####### leaving the map layers, if user wants to 'flatten out' the bumped layer 
####### they can make visible the design or inner ring layers for a different look

	if legend_iv==0:
		invert=0
	if legend_iv==1:
		invert=1	

	#pdb.plug_in_bump_map(image,coin,legend, 150,45,bh_3,0,0,0,0.58,1,invert,0)
	pdb.plug_in_bump_map(image,coin,legend, 145,45,bh_3,0,0,0,0.58,1,invert,0)
	pdb.plug_in_bump_map(image,coin,legend, 145,45,bh_3,-1,-1,0,0.58,1,invert,0)	


	####### Cleanup and exit.

	pdb.gimp_image_select_item(image,0,legend)
	pdb.gimp_context_set_foreground(fg_1)
	pdb.gimp_edit_fill(legend,0)
	pdb.gimp_selection_none(image)
	#set visibles

	pdb.gimp_item_set_visible(coin,True)
	pdb.gimp_item_set_visible(rim,True)

	if ct==3:
		w = int(diameter + 2*margin)
		h = int(diameter + 2*margin)
		add = gimp.Layer(image, "", w, h, RGBA_IMAGE, 100, 0)
		image.add_layer(add, -1)
		center = pdb.gimp_text_fontname(image, None, 0, 0, text_c, 0, 1, fs_c, PIXELS, font_c)
		pdb.gimp_text_layer_set_justification (center, 2)
		pdb.gimp_text_layer_set_line_spacing(center, line_sp)
		pdb.gimp_context_set_foreground((fg_1))
		##pdb.plug_in_gauss_rle2(image,center,2,2)
		gegl_batch(image,center,'gaussian-blur std-dev-x=1 std-dev-y=1')		
		pdb.gimp_image_select_item(image,0,center)
		pdb.gimp_edit_fill(center,0)

		wh = pdb.gimp_drawable_width(center)
		hh = pdb.gimp_drawable_height(center)
		centerx=w/2
		centery=h/2

		adw=wh/2
		adh=hh/2
		sw=centerx - adw
		sh=centery - adh		
		pdb.gimp_layer_set_offsets(center, sw, sh)
		center.name = 'Center'
		layer = pdb.gimp_image_merge_down(image, center, 2)
		pdb.gimp_layer_flatten(layer)
	
		pdb.gimp_item_set_visible(layer, 0)
		pdb.gimp_image_set_active_layer(image, coin)
		pdb.gimp_selection_none(image)
	#pdb.plug_in_bump_map(image, drawable, bumpmap, azimuth, elevation, depth, xofs, yofs, waterlevel, ambient, compensate, invert, type)
	if legend_c==0:
		invert_c=0
	if legend_c==1:
		invert_c=1
	if ct!=0 or ct==4:		
	#pdb.plug_in_bump_map(image,coin,layer, 150,45,bh_c,0,0,0,0.58,1,invert_c,0)
		pdb.plug_in_bump_map(image,coin,layer, 145,45,bh_c,0,0,0,0.58,1,invert_c,0)
		pdb.plug_in_bump_map(image,coin,layer, 145,45,bh_c,-1,-1,0,0.58,1,invert_c,0)
    
	if make_r:
		ring=image.new_layer()
		ring.name = 'Ring'    
		o_rad = pdb.gimp_vectors_new(image,"outer_ring")
		pdb.gimp_image_insert_vectors(image, o_rad, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(o_rad, margin+diameter/2, margin+diameter/2, out_rad, out_rad, 0)
		pdb.gimp_image_select_item(image,2,o_rad)
		
		i_rad = pdb.gimp_vectors_new(image,"inner_ring")
		pdb.gimp_image_insert_vectors(image, i_rad, None, -1)
		pdb.gimp_vectors_bezier_stroke_new_ellipse(i_rad, margin+diameter/2, margin+diameter/2, in_rad, in_rad, 0)
		pdb.gimp_image_select_item(image,1,i_rad)
		
# Deprecated: pdb.gimp_vectors_to_selection(vectors, operation, antialias, feather, feather_radius_x, feather_radius_y)        
# Use:        pdb.gimp_image_select_item(image, operation[CHANNEL-OP-ADD (0), CHANNEL-OP-SUBTRACT (1), CHANNEL-OP-REPLACE (2), CHANNEL-OP-INTERSECT (3)], item)  

		pdb.gimp_context_set_foreground((fg_ring))
		pdb.gimp_edit_fill(ring,0)
		pdb.gimp_selection_none(image)
		##pdb.plug_in_gauss_rle2(image,pdb.gimp_image_get_layer_by_name(image, "Ring"),2,2) #medium blur for less alias on bump.
		gegl_batch(image,pdb.gimp_image_get_layer_by_name(image, "Ring"),'gaussian-blur std-dev-x=1 std-dev-y=1')
		#pdb.plug_in_bump_map(image,coin,ring, 150,45,bh_r,0,0,0,0.58,1,invert_r,0)
		pdb.plug_in_bump_map(image,coin,ring, 145,45,bh_r,0,0,0,0.58,1,invert_r,0)
		pdb.plug_in_bump_map(image,coin,ring, 145,45,bh_r,-1,-1,0,0.58,1,invert_r,0)		
	if make_st:	

		stars = gimp.Layer(image, "Stars", int(diameter+2*margin), int(diameter+2*margin), RGBA_IMAGE, 100, 0)
		image.add_layer(stars, 1)
		pdb.gimp_edit_clear(stars)
		pdb.gimp_image_set_active_layer(image,stars)
		stars.name = 'Stars'
		pdb.gimp_context_set_foreground(fg_stars)
		pdb.gimp_context_set_brush('2. Star')
		pdb.gimp_context_set_brush_size(size_star)
		pdb.gimp_context_set_brush_default_hardness()
		pdb.gimp_context_set_brush_spacing(0.01)
		pdb.gimp_context_set_paint_method("gimp-paintbrush")
		pdb.gimp_context_set_brush_aspect_ratio(1.5)
		pdb.gimp_context_set_brush_angle(17.5)
		pdb.gimp_context_set_paint_method("gimp-paintbrush")
		pdb.gimp_context_set_dynamics("Dynamics Off")  

		pdb.gimp_paintbrush_default(stars,2,(int(margin+diameter/2),int(margin+diameter/2 - radius_s)))
		w=int(margin+diameter/2)
		h=int(margin+diameter/2)
		mura_cc(image,stars,w,h,many_star)
		pdb.gimp_image_raise_item(image, stars)
		st=pdb.gimp_image_merge_down(image, pdb.gimp_image_get_layer_by_name(image, 'Many Stars'), 2)
		##pdb.plug_in_gauss_rle2(image,st,2,2)
		#gegl_batch(image,st,'gaussian-blur std-dev-x=1 std-dev-y=1')		
		if make_r:
			#pdb.plug_in_bump_map(image,ring,st, 150,45,bhs,0,0,0,0.58,1,invert_s,0)
			pdb.plug_in_bump_map(image,ring,st, 145,45,bhs,0,0,0,0.58,1,invert_s,0)
			pdb.plug_in_bump_map(image,ring,st, 145,45,bhs,-1,-1,0,0.58,1,invert_s,0)
		else:
		
			#pdb.plug_in_bump_map(image,coin,st, 150,45,bhs,0,0,0,0.58,1,invert_s,0)
			pdb.plug_in_bump_map(image,coin,st, 145,45,bhs,0,0,0,0.58,1,invert_s,0)
			pdb.plug_in_bump_map(image,coin,st, 145,45,bhs,-1,-1,0,0.58,1,invert_s,0)
		#gegl_batch(image, st,'emboss type=bumpmap azimuth=275 elevation=30 depth=50')
		pdb.gimp_item_set_visible(st, 1)
        
#######################################################################################
	# add for "Preview"		
	view = pdb.gimp_layer_new_from_visible(image, image, "Preview")
	pdb.gimp_image_insert_layer(image, view, None, 0)
	pdb.gimp_image_set_active_layer(image,view)
	pdb.gimp_displays_flush() 
#######################################################################################
	
	# Set end undo point

    
hlp=" Plugin Based On Conbagui's Gold Coin Tutorial"    \
+ "\n and BA_Conbagui_Coin.py by Brian Andersen."   \
+ "\n\n If You want to use the active layer to decorate a coin,\n select a size similar to the coin's diameter"   \
+ "\n (but You can also choose to scale to coin size)."	\
+ "\n\n Resizing method 'Preview':\n 1. Disable preview.\n 2. Drag the corner.\n 3. Turn on the preview again."	\
+ "\n\n Only the size of the initial (active) layer is displayed, so the preview\n may show an image slightly different from the final result."	\
+ "\n\n Wish You have fun with this plugin:\n\n MareroQ."


# This is the plugin registration function
register(
	"MrQ_Coin_Creator",    
	" This Plugin Create Coin (as New Layers).Ver.7.0 ",   
	"Requirments show at top",
	"Brian Andersen/MrQ", 
	"GPL", 	
	"2021",
	"Coin Creator...", 
	"RGB*", 
	[
		(PF_IMAGE,    "image",         "Input image", None),
		(PF_DRAWABLE, "drawable",      "Input drawable", None), 
		(PF_SPINNER,  "diameter",      "Diameter Size: ", 500, (10, 3500, 1 )),
		(PF_SPINNER,  "margin",        "Add a blank border\n (edge width): ", 10, (0, 100, 1 )),	  
		(PF_COLOR,    "fg_1",          "Pick Main Color: ",(206,187,120)),
		(PF_COLOR,    "fg_2",          "Pick Rim Color: ",(138,124, 76)),
		(PF_SPINNER,  "rim_thickness", "Rim Thickness: ", 5, (0, 500, 1)),
		(PF_TOGGLE,   "edg",           "Make Rim Grooves? ", 0, ["DON'T USE","Yes"]),
		(PF_SPINNER,  "marksCount",    "Grooves Count: ", 250, (10, 1500, 1)),
		(PF_SPINNER,  "marksCount",    "Grooves Width: ", 1, (1, 90, 1)),        
		(PF_FONT,     "font_t",    "Top Legend Font: ", "Sans-serif",0,{'tab':1}),
		(PF_SPINNER,  "fs_t",      "Top Legend Font Size: ", 50, (10, 1500, 1),{'tab':1}),	  
		(PF_SPINNER,  "ex_t",      "Extra Spacing Top Font:", 3, (0, 150, 1 ),{'tab':1}),	  
		(PF_STRING,   "text_t",    "Top Legend Text: ", "GIMPSCRIPTS.NET",0,{'tab':1}),	
		(PF_OPTION,   "ct",        "Make Center? ", 3, [" DON'T USE ", " Use Active Layer "," Use Scaleded Layer\n (60% coin size) ", " Use Font ", " Fill the entire coin \n (with the layer) "],{'tab':2}),
		(PF_FONT,     "font_c",    "Center Font: ", "Wingdings ",0,{'tab':2}),
		(PF_SPINNER,  "fs_c",      "Font Size:", 270, (10, 1500, 1),{'tab':2}),
		(PF_TEXT,     "text_c",    "Center Text: ", "A",0,{'tab':2}),
		(PF_SPINNER,  "line_sp",   "Line Spacing:", 0, ( -500, 500, 1,),{'tab':2}),	
		(PF_FONT,     "font_b",    "Bottom Font: ", "Sans-serif",0,{'tab':3}),
		(PF_SPINNER,  "fs_b",      "Font Size: ",      50, (10, 1500, 1),{'tab':3}),		  
		(PF_SPINNER,  "ex_b",      "Extra Spacing: ", 3, (0, 150, 1 ),{'tab':3}),	  
		(PF_OPTION,   "type_b",    "Text Type: ", 0, ["From Left to Right",  "From Right to Left","Only Top Text in a Circle"],{'tab':3}),
		(PF_STRING,   "text_b",    "Bottom Legend Text:", "★★★★ 2021 ★★★★",0,{'tab':3}),
		(PF_TOGGLE,   "make_r",    "Make Ring? ",0, ["DON'T USE","YES"],{'tab':4}),
		(PF_COLOR,    "fg_ring",   "Ring Color: ",(206,187,120),0,{'tab':4}),		
		(PF_SPINNER,  "out_rad",   "Outer Radius: ", 170, (1, 2000, 1),{'tab':4}),
		(PF_SPINNER,  "in_rad",    "Inner Radius: ", 130, (1, 2000, 1),{'tab':4}),	
		(PF_TOGGLE,   "make_st",   "Make Stars? ",0, ["DON'T USE","YES"],{'tab':4}),
		(PF_COLOR,    "fg_stars",  "Stars Color: ",(138,124, 76),0,{'tab':4}),			
		(PF_SPINNER,  "size_star", "Size Star: ", 30, (1, 200, 1),{'tab':4}),
		(PF_SPINNER,  "radius_s",  "Distance from\n the center: ", 150, (1, 1000, 1),{'tab':4}),		
		(PF_SPINNER,  "many_star", "Many Stars: ", 7, (1, 150, 1),{'tab':4}),
		(PF_TITLE  ,  "code_c"  ,  "<b>\n			     Center Legend.</b>",(0.0,0.0), None,{'iniCol':1,'cols':2,'tab':5}),
		(PF_SPINNER,  "bh_c",      "Bump Height: ", 25, (1, 65, 1),{'tab':5}),		
		(PF_TOGGLE,   "legend_c",  "Bump Invert? ",1, [" YES "," DON'T USE "],{'tab':5}),
		(PF_TITLE  ,  "code_t"  ,  "<b>\n			  Bottom and Top Legend.</b>",(0.0,0.0), None,{'iniCol':1,'cols':2,'tab':5}),
		(PF_SPINNER,  "bh_3",      "Bump Height: ", 15, (1, 65, 1),{'tab':5}),
		(PF_TOGGLE,   "legend_iv", "Bump Invert? ",0, [" DON'T USE "," YES "],{'tab':5}),
		(PF_TITLE  ,  "code_r"  ,  "<b>\n			  		   Ring.</b>",(0.0,0.0), None,{'iniCol':1,'cols':2,'tab':5}),
		(PF_SPINNER,  "bh_r",      "Bump Height: ", 15, (1, 65, 1),{'tab':5}),	
		(PF_TOGGLE,   "invert_r",  "Bump Invert? ",0, [" DON'T USE "," YES "],{'tab':5}),
		(PF_TITLE  ,  "code_s"  ,  "<b>\n			  		  Stars.</b>",(0.0,0.0), None,{'iniCol':1,'cols':2,'tab':5}),		
		(PF_SPINNER,  "bhs",       "Bump Height : ",  15, (1, 65, 1),{'tab':5}),        
 		(PF_TOGGLE,   "invert_s",  "Bump Invert? ",0, [" DON'T USE "," YES "],{'tab':5}),
		
		(PF_TITLE  , "help"  , hlp ,(10,10), None           ,{'tab':6}),		
		(PF_TAB    , "ptab"    , "tabs" , 0, (" Coin "," Top ", " Center "," Bottom "," Ring & Stars "," Bump-Map "," Help "),{'newLine':'1','tab':6} ),       
	], 
	[],
	MrQ_Coin_Creator,
	menu="<Image>/Filters/Render",
	domain=("gimp20", gimp.locale_directory)
	)

main()
